@@ -23,16 +23,23 @@ centrality <- function(graph,alpha=1,posfun=abs)
     }
   }
   
+  # Remove diagonal:
+  if (any(diag(W)!=0))
+  {
+    message("Self-loops are not included in centrality analysis.")
+    diag(W) <- 0 
+  }
+  
   ## Compute adjacency:
   X <- 1L * (W!=0)
   
   ## Compute default measures:
   UnweightedDegreesOut <- rowSums(X)
-  WeightedDegreesOut <- rowSums(W)
+  WeightedDegreesOut <- rowSums(posfun(W))
   CombinedDegreesOut <- UnweightedDegreesOut^(1-alpha) * WeightedDegreesOut^alpha
   
   UnweightedDegreesIn <- colSums(X)
-  WeightedDegreesIn <- colSums(W)
+  WeightedDegreesIn <- colSums(posfun(W))
   CombinedDegreesIn <- UnweightedDegreesIn^(1-alpha) * WeightedDegreesIn^alpha
   
   # Compute shortest distance using Dijkstra (code based on pseudo code on Wikipedia)
